import use_psyco, section, tagger, language_model, config, reader, utils, maxent, decoder, stopwords, stem_model
import sys, getopt


def build_lexical_language_model(root_section, order):

    utils.p( "* Building lexical language model..." )
    utils.delete_lm_pickle ()
    def sentence_text_generator():
        for sec in root_section.get_descendants():
            for sentence in sec.body:
                #yield sentence.text.lower()
                yield sentence.get_tagged_text ()
    lm = language_model.LanguageModel()
    lm.train(sentence_text_generator(), order)
    utils.save_lm( lm )

def build_title_tag_language_model(root_section, order, start, end):
    
    utils.p( "* Building title tag language model..." )
    utils.delete_tag_lm_pickle ()
    lm = language_model.LanguageModel()

    def title_tag_text_generator():
        sections = root_section.get_descendants()[int(start):int(end) + 1]
        for sec in sections:
            yield ' '.join( map(lambda x: x.pos_tag, sec.title.words) )

    tagger.tag_titles_from_text( root_section )
    lm.train(title_tag_text_generator(), order)
    utils.save_tag_lm(lm)

def tag_corpus( root_section ):
    utils.p( "* Tagging corpus..." )

    tagger.tag_corpus( root_section )
    root_section.build_np_counts ()

    utils.p( "* Saving corpus..." )
    utils.save_section( root_section )

def tag_from_file( root_section ):
    
    utils.p( "* Tagging from file..." )
    tagger.tag_from_file( root_section )
    
    utils.p( "* Saving corpus..." )
    utils.save_section( root_section )

def load_dataset():
    utils.p( "* Reading corpus..." )
    try:
        utils.p("* Loading from pickle...")
        return utils.load_section()
    except:
        utils.p("* Loading from XML...")
        return reader.read_corpus()

def build_stopwords_list( root_section, number_of_stopwords, start_sec, end_sec ):
    utils.p("* Building stopwords list...")
    stopwords_set = stopwords.get_stopwords_set( root_section.get_descendants()[int(start_sec):int(end_sec)], int (number_of_stopwords))
    utils.delete_stopwords_set()
    utils.save_stopwords_set( stopwords_set )

    utils.p("** You should now manually tune the stopwords list.")
    
def train( root_section, start_sec, end_sec, window_size ):

    window_size = int( window_size )

    lm = utils.load_lm()
    tag_lm = utils.load_tag_lm()
    stopwords_set = utils.load_stopwords_set()
    s_model = utils.load_stem_model()
    
    m = maxent.Model( root_section, window_size, tag_lm, lm, stopwords_set, s_model )

    utils.p(utils.time())
    previous_sec = None
    for sec in root_section.get_descendants()[int(start_sec):int(end_sec) + 1]:
        m.add_training_features_section( sec, previous_sec )
        previous_sec = sec

    utils.p(utils.time())
    m.train()
    utils.p(utils.time())    

def decode( root_section, length, beam_size, start_sec, end_sec, window_size):
    
    beam_size = int( beam_size )
    length = int( length )
    window_size = int( window_size)
    lm = utils.load_lm()
    tag_lm = utils.load_tag_lm()
    stopwords_set = utils.load_stopwords_set()
    s_model = utils.load_stem_model()
    
    m = maxent.Model( root_section, window_size, tag_lm, lm, stopwords_set, s_model )

    previous_sec = None
    current_section = int (start_sec)
    for sec in root_section.get_descendants()[int(start_sec):int(end_sec) + 1]:
        gen_length = length
        if length == 0: 
			#gen_length = len(sec.title.words)
            gen_length = sec.title.text.count (' ') - 2
        
        utils.p( str (current_section) )
        utils.p( 'ACT : ' + sec.title.text ) 
        utils.p( 'GEN : ' + ' '.join( decoder.beam_search(sec, previous_sec, gen_length, beam_size, window_size, m) ) )
        previous_sec = sec
        current_section += 1

def stem_corpus( root_section ):
    sm = stem_model.StemModel( root_section )
    utils.save_stem_model( sm )

def main():
    
    try:
        opts, args = getopt.getopt (sys.argv[1:], "", ["mode=", "dataset=",  "order=", "length=", "beam_size=", "start=", "end="])

    except getopt.GetoptError:
        print "Invalid arguments supplied"
        sys.exit(2)

    def opt(name):
        key = "--"+name
        for option, val in opts:
            if option == key:
                return val
        return None

    assert(opt("mode") != None)

    # Set global dataset
    config.DATASET_NAME = opt("dataset")
    if config.DATASET_NAME == None:
        config.DATASET_NAME = "algorithms"
        utils.p ("dataset not specified in command line, using 'algorithms' by default")

    arg_mode = opt("mode")
    arg_order = opt("order")
    
    if arg_mode == "tag" or arg_mode == "tag_from_file":
        utils.delete_section_pickle ()

    # Load corpus from pickle or file
    root_section = load_dataset()

    if arg_mode == "lm":
        assert(arg_order != None)
        build_lexical_language_model( root_section, arg_order )
        
    elif arg_mode == "taglm":
        assert(arg_order != None)
        build_title_tag_language_model( root_section, arg_order, opt("start"), opt("end") )
        
    elif arg_mode == "tag":
        tag_corpus( root_section )

    elif arg_mode == "stem":
        stem_corpus( root_section )

    elif arg_mode == "tag_from_file":
        tag_from_file( root_section )

    elif arg_mode == "stopwords":
        build_stopwords_list( root_section, config.NUMBER_OF_STOPWORDS, opt("start"), opt("end") )
        
    elif arg_mode == "create_models":
        assert (arg_order != None)
        tag_from_file (root_section)
        build_lexical_language_model( root_section, arg_order )
        build_title_tag_language_model( root_section, arg_order, opt("start"), opt("end") )
        build_stopwords_list( root_section, config.NUMBER_OF_STOPWORDS, opt("start"), opt("end") )

    elif arg_mode == "train":
        train( root_section, opt("start"), opt("end"), arg_order )

    elif arg_mode == "decode":
        decode( root_section, opt("length"), opt("beam_size"), opt("start"), opt("end"), arg_order )
    else:
        raise Exception( "Error: Invalid Mode ("+arg_mode+")" )

if __name__ == '__main__':
    sys.exit( main( ) )
